/*
* This is a simple example use of ofArduino
*
* ofArduino currently only supports the standard Arduino boards
* (UNO, Duemilanove, Diecimila, NG, and other boards based on the
* ATMega168 or ATMega328 microcontrollers
* The Arduio FIO and Arduino Mini should also work.
* The Arduino MEGA and other variants based on microcontrollers
* other than the ATMega168 and ATMega328 are not currently supported.
*
* To use this example, open Arduino (preferably Arduino 1.0) and
* navigate to File -> Examples -> Firmata and open StandardFirmata.
* Compile and upload StandardFirmata for your board, then close
* the Arduino application and run this application.
*
* If you have a servo attached, press the left arrow key to rotate
* the servo head counterclockwise and press the right arrow key to
* rotate the servo head clockwise.
*
* Clicking the mouse over any part of the application will turn the
* on-board LED on and off.
*
*/

#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup() {

	ofSetVerticalSync(true);
	ofSetFrameRate(60);

	ofBackground(255, 255, 255);


	buttonState = "digital pin:";
	potValue = "analog pin:";

	font.load("franklinGothic.otf", 20);
	smallFont.load("franklinGothic.otf", 14);


	ardUno.connect("COM4", 57600);
	ardUno.sendFirmwareVersionRequest();

	ardFio.connect("COM6", 57600);
	ardFio.sendFirmwareVersionRequest();

	mode = 0;


	// listen for EInitialized notification. this indicates that
	// the arduino is ready to receive commands and it is safe to0
	// call setupArduino()
	ofAddListener(ardUno.EInitialized, this, &ofApp::setupArduino);
	//	ofAddListener(ardFio.EInitialized, this, &ofApp::setupArduino);
	bSetupArduino = false;	// flag so we setup arduino when its ready, you don't need to touch this :)

							// GRAPHICS
							// Hand
							//		ofSetLogLevel(OF_LOG_VERBOSE);
	ofDisableArbTex(); // we need GL_TEXTURE_2D for our models coords.
	bAnimate = false;
	bAnimateMouse = false;
	animationPosition = 0;

	model.loadModel("Hand.3ds", true);
	model.setPosition(ofGetWidth() * 0.5, (float)ofGetHeight() * 0.5, 0);

	model.setLoopStateForAllAnimations(OF_LOOP_NORMAL);
	model.playAllAnimations();
	if (!bAnimate) {
		model.setPausedForAllAnimations(true);
	}

	// Sphere
	bFill = true;
	bWireframe = true;
	bDrawNormals = false;
	bDrawAxes = false;
	bDrawLights = false;
	bInfoText = false;
	bMousePressed = false;
	bSplitFaces = false;

	float width = ofGetWidth() * .12;
	float height = ofGetHeight() * .12;


	// Rotations
	rotateX = 0.0f;
	rotateY = 0.0f;
	rotateZ = 0.0f;



	// Lights
	ofSetSmoothLighting(true);
	pointLight.setDiffuseColor(ofFloatColor(.85, .85, .55));
	pointLight.setSpecularColor(ofFloatColor(1.f, 1.f, 1.f));

	pointLight2.setDiffuseColor(ofFloatColor(238.f / 255.f, 57.f / 255.f, 135.f / 255.f));
	pointLight2.setSpecularColor(ofFloatColor(.8f, .8f, .9f));

	pointLight3.setDiffuseColor(ofFloatColor(19.f / 255.f, 94.f / 255.f, 77.f / 255.f));
	pointLight3.setSpecularColor(ofFloatColor(18.f / 255.f, 150.f / 255.f, 135.f / 255.f));


	models.push_back("Hand.3ds");
	models.push_back("head.OBJ");

}

//--------------------------------------------------------------
void ofApp::update() {

	updateArduino();

	model.update();

	mesh = model.getCurrentAnimatedMesh(0);


	// Lights
	pointLight.setPosition((ofGetWidth()*.5) + cos(ofGetElapsedTimef()*.5)*(ofGetWidth()*.3), ofGetHeight() / 2, 500);
	pointLight2.setPosition((ofGetWidth()*.5) + cos(ofGetElapsedTimef()*.15)*(ofGetWidth()*.3),
		ofGetHeight()*.5 + sin(ofGetElapsedTimef()*.7)*(ofGetHeight()), -300);

	pointLight3.setPosition(
		cos(ofGetElapsedTimef()*1.5) * ofGetWidth()*.5,
		sin(ofGetElapsedTimef()*1.5f) * ofGetWidth()*.5,
		cos(ofGetElapsedTimef()*.2) * ofGetWidth()
		);

	ofSetWindowTitle("Framerate: " + ofToString(ofGetFrameRate(), 0));


}

//--------------------------------------------------------------
void ofApp::setupArduino(const int & version) {

	// remove listener because we don't need it anymore
	ofRemoveListener(ardUno.EInitialized, this, &ofApp::setupArduino);
	// it is now safe to send commands to the Arduino
	bSetupArduino = true;

	// print firmware name and version to the console
	ofLogNotice() << ardUno.getFirmwareName();
	ofLogNotice() << "firmata v" << ardUno.getMajorFirmwareVersion() << "." << ardUno.getMinorFirmwareVersion();

	ofLogNotice() << ardFio.getFirmwareName();
	ofLogNotice() << "firmata v" << ardFio.getMajorFirmwareVersion() << "." << ardFio.getMinorFirmwareVersion();

	// ---- Arduino UNO pins ----

	// Accelerometer calibration button
	ardUno.sendDigitalPinMode(calibPin, ARD_INPUT);

	// Buttons
	ardUno.sendDigitalPinMode(but1Pin, ARD_INPUT);
	ardUno.sendDigitalPinMode(but2Pin, ARD_INPUT);
	ardUno.sendDigitalPinMode(but3Pin, ARD_INPUT);


	// Sound
	ardUno.sendAnalogPinReporting(soundInput, ARD_ANALOG);

	// Potentiometers
	ardUno.sendAnalogPinReporting(pot1Pin, ARD_ANALOG);
	ardUno.sendAnalogPinReporting(pot2Pin, ARD_ANALOG);
	ardUno.sendAnalogPinReporting(pot3Pin, ARD_ANALOG);


	// ---- Arduino FIO pins ----

	// Bends
	ardFio.sendAnalogPinReporting(bendOneInput, ARD_ANALOG);
	ardFio.sendAnalogPinReporting(bendTwoInput, ARD_ANALOG);
	ardFio.sendAnalogPinReporting(bendThreeInput, ARD_ANALOG);

	// Accelerometer: X, Y, Z
	ardFio.sendAnalogPinReporting(xInput, ARD_ANALOG);
	ardFio.sendAnalogPinReporting(yInput, ARD_ANALOG);
	ardFio.sendAnalogPinReporting(zInput, ARD_ANALOG);

	ardFio.sendDigitalPinMode(fioLed, ARD_OUTPUT);


	// Listen for changes on the digital and analog pins
	ofAddListener(ardUno.EDigitalPinChanged, this, &ofApp::digitalPinChanged);
	ofAddListener(ardUno.EAnalogPinChanged, this, &ofApp::analogPinChanged);
}

//--------------------------------------------------------------
void ofApp::updateArduino() {

	// update the arduino, get any data or messages.
	// the call to ardUno.update() is required
	ardUno.update();
	ardFio.update();

	// do not send anything until the arduino has been set up
	if (bSetupArduino) {
		/* ---- Arduino UNO functionalities ---- */
		/* Accelerometer */
		int xRaw = ReadAxis(xInput);
		int yRaw = ReadAxis(yInput);
		int zRaw = ReadAxis(zInput);
		// report plz
		cout << "xRaw: " << xRaw << ", yRaw:" << yRaw << ", zRaw: " << zRaw << "\n";
		if (ardUno.getDigital(calibPin)) // If calibration button is HIGH
		{
			calib = true;
			calibrateAccel(xRaw, yRaw, zRaw);
		}
		else { calib = false; }
		long xScaled = ofMap(xRaw, xRawMin, xRawMax, -1000, 1000);
		long yScaled = ofMap(yRaw, yRawMin, yRawMax, -1000, 1000);
		long zScaled = ofMap(zRaw, zRawMin, zRawMax, -1000, 1000);
		// re-scale to fractional Gs (-1.00 - 1.00)
		xAccel = xScaled / 1000.0;
		yAccel = yScaled / 1000.0;
		zAccel = zScaled / 1000.0;
		// report values
		cout << "xAccel: " << xAccel << ", yAccel:" << yAccel << ", zAccel: " << zAccel << "\n";


		/* Pots and all */
		pot1Val = ardUno.getAnalog(pot1Pin);
		pot2Val = ardUno.getAnalog(pot2Pin);
		pot3Val = ardUno.getAnalog(pot3Pin);
		but1Val = ardUno.getDigital(but1Pin);
		but2Val = ardUno.getDigital(but2Pin);
		but3Val = ardUno.getDigital(but3Pin);

		/* Sound */
		soundVal = ardUno.getAnalog(soundInput);

		/* ---- Arduino FIO functionalities ---- */

		bendOne = ardFio.getAnalog(bendOneInput);
		bendTwo = ardFio.getAnalog(bendTwoInput);
		bendThree = ardFio.getAnalog(bendThreeInput);

		ardFio.sendDigital(fioLed, ARD_HIGH);
		// 20ms delay just for kicks
		//	Sleep(200);
		ardFio.sendDigital(fioLed, ARD_LOW);

	}
	else {
		ardUno.sendFirmwareVersionRequest();
		ardFio.sendFirmwareVersionRequest();
	}
}

// digital pin event handler, called whenever a digital pin value has changed
// note: if an analog pin has been set as a digital pin, it will be handled
// by the digitalPinChanged function rather than the analogPinChanged function.

//--------------------------------------------------------------
void ofApp::digitalPinChanged(const int & pinNum) {
	// do something with the digital input. here we're simply going to print the pin number and
	// value to the screen each time it changes
	buttonState = "digital pin: " + ofToString(pinNum) + " = " + ofToString(ardUno.getDigital(pinNum));
}

// analog pin event handler, called whenever an analog pin value has changed

//--------------------------------------------------------------
void ofApp::analogPinChanged(const int & pinNum) {
	// do something with the analog input. here we're simply going to print the pin number and
	// value to the screen each time it changes
	//   potValue = "analog pin: " + ofToString(pinNum) + " = " + ofToString(ardUno.getAnalog(pinNum));
	potValue = "pin: " + ofToString(0) + " = " + ofToString(ardUno.getAnalog(0)) +
		"\npin: " + ofToString(1) + " = " + ofToString(ardUno.getAnalog(1)) +
		"\npin: " + ofToString(2) + " = " + ofToString(ardUno.getAnalog(2)) +
		"\npin: " + ofToString(3) + " = " + ofToString(ardUno.getAnalog(3)) +
		"\npin: " + ofToString(4) + " = " + ofToString(ardUno.getAnalog(4));
}


//--------------------------------------------------------------

void ofApp::draw() {

	int bgColor = 0;
	if (soundVal > 200) { bgColor = 255; }
	ofBackground(bgColor, bgColor, bgColor);
	ofEnableAlphaBlending();
	ofSetColor(255, 255, 255);


//	if (!bSetupArduino) {
//		font.drawString("Arduino not ready...\n", 20, 40);
//	}
//	else {
		// ---- ---- ONLY DRAW IF ARDUINO IS SET UP ---- ----

		// Information

		if (bInfoText) {
			ofDrawBitmapString("fps: " + ofToString(ofGetFrameRate(), 2), 700, 15);

			ofSetColor(255, 255, 255, 255);
			stringstream ss;
			ss << "Framerate: " << ofToString(ofGetFrameRate(), 0) << "\n";
			ss << "(f): Toggle Fullscreen" << endl << "(s): Draw Solid Shapes" << endl << "(w): Draw Wireframes" << endl;
			ss << "(z): Split Faces " << bSplitFaces << endl;
			ss << "(a): Draw Axes" << endl << "(l): Render lights" << endl << "(t): Info Text" << endl;

			ofDrawBitmapString(ss.str().c_str(), 20, 300);

			smallFont.drawString("Bendz: 1: " + ofToString(bendOne) + "\n" +
				"2: " + ofToString(bendTwo) + "\n" +
				"3: " + ofToString(bendThree) + "\n"
				, 300, 40);

			// Report Arduino values
			smallFont.drawString(potValue + "\n" + buttonState + "\nsending pwm: " + ofToString((int)(128 + 128 * sin(ofGetElapsedTimef()))), 20, 40);
			// Calibrate or report XYZ
			if (calib) {
				smallFont.drawString("Calibrating...", 20, 500);
			}
			else {
				smallFont.drawString("x: " + ofToString(xAccel) + "\ny: " + ofToString(yAccel) + "\nz: " + ofToString(zAccel), 20, 500);
			}

			smallFont.drawString("Sound value: " + ofToString(soundVal), 500, 40);

			smallFont.drawString("Potentiometers: \n1: " + ofToString(pot1Val) + "\n2: " + ofToString(pot2Val) + "\n3: " + ofToString(pot3Val), 500, 60);

			smallFont.drawString("Buttons: \n1: " + ofToString(but1Val) + "\n2: " + ofToString(but2Val) + "\n3: " + ofToString(but3Val), 650, 60);

		}


		ofEnableBlendMode(OF_BLENDMODE_ALPHA);

		ofEnableDepthTest();

#ifndef TARGET_PROGRAMMABLE_GL    
		glShadeModel(GL_SMOOTH);
#endif

		ofEnableLighting();
		pointLight.enable();
		pointLight2.enable();
		pointLight3.enable();
		//	ofEnableSeparateSpecularLight();
		
		// Graphics

		float spinX = sin(ofGetElapsedTimef()*((.35f)));
		float spinY = sin(ofGetElapsedTimef()*((.075f)));

		if (bMousePressed) {
			spinX = spinY = 0.0f;
		}

		float middleFinger = ofMap(bendThree, 200, 500, -100, 100);
		ofTranslate(0, 0, middleFinger);

		if (bWireframe) {
			ofPushMatrix();
			ofTranslate(model.getPosition().x, model.getPosition().y, 0);
			ofRotate(-mouseX, 0, 1, 0);
			ofTranslate(-model.getPosition().x, -model.getPosition().y, 0);

			model.drawFaces();
			ofPopMatrix();
		}
		else {
#ifndef TARGET_PROGRAMMABLE_GL    
			glEnable(GL_NORMALIZE);
#endif
			ofPushMatrix();
			ofTranslate(model.getPosition().x, model.getPosition().y, 0);
			ofRotate(-mouseX, 0, 1, 0);
			ofTranslate(-model.getPosition().x, -model.getPosition().y, 0);

			ofxAssimpMeshHelper & meshHelper = model.getMeshHelper(0);

			ofMultMatrix(model.getModelMatrix());
			ofMultMatrix(meshHelper.matrix);

			ofMaterial & material = meshHelper.material;
			if (meshHelper.hasTexture()) {
				meshHelper.getTextureRef().bind();
			}
			material.begin();
			mesh.drawWireframe();
			material.end();
			if (meshHelper.hasTexture()) {
				meshHelper.getTextureRef().unbind();
			}

			ofPopMatrix();
		}

		ofDisableDepthTest();
		pointLight.disable();
		pointLight2.disable();
		pointLight3.disable();
		ofDisableLighting();
//		ofDisableSeparateSpecularLight();

		// ---- ---- ---- END DRAW IF ARDUINO ---- ---- ----
//	}
}

//--------------------------------------------------------------
void ofApp::keyPressed(int key) {

	switch (key) {
	case 'f':
		ofToggleFullscreen();
		break;
	case 's':
		bFill = !bFill;
		break;
	case 'w':
		bWireframe = !bWireframe;
		break;
	case 'a':
		bDrawAxes = !bDrawAxes;
		break;
	case 'l':
		bDrawLights = !bDrawLights;
		break;
	case 't':
		bInfoText = !bInfoText;
		break;
	case OF_KEY_RIGHT:
		mode++;
		if (mode > models.size()-1) { mode = 0; }
		model.loadModel(models[mode], true);
		model.setPosition(ofGetWidth() * 0.5, (float)ofGetHeight() * 0.5, 0);
	break;
	case OF_KEY_LEFT:
		mode--;
		if (mode < 0) { mode = models.size()-1; }

		model.loadModel(models[mode], true);
		model.setPosition(ofGetWidth() * 0.5, (float)ofGetHeight() * 0.5, 0);
		break;
	}
}

//--------------------------------------------------------------
void ofApp::keyReleased(int key) {

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y) {

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button) {

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button) {
	// turn on the onboard LED when the application window is clicked
	ardUno.sendDigital(13, ARD_HIGH);
	ardFio.sendDigital(13, ARD_HIGH);

	cout << size(models);

	/*
	ardFio.connect("COM6", 57600);
	ardFio.sendFirmwareVersionRequest();
	ofAddListener(ardFio.EInitialized, this, &ofApp::setupArduino);
	*/

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button) {
	// turn off the onboard LED when the application window is clicked
	ardUno.sendDigital(13, ARD_LOW);
	ardFio.sendDigital(13, ARD_LOW);
}

//--------------------------------------------------------------
void ofApp::mouseEntered(int x, int y) {

}

//--------------------------------------------------------------
void ofApp::mouseExited(int x, int y) {

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h) {
	model.setPosition(ofGetWidth() * 0.5, (float)ofGetHeight() * 0.5, 0);
}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg) {

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo) {

}

//
// Find the extreme raw readings from each axis
//
void ofApp::calibrateAccel(int xRaw, int yRaw, int zRaw)
{
	printf("Calibrate\n");
	if (xRaw < xRawMin)
	{
		xRawMin = xRaw;
	}
	if (xRaw > xRawMax)
	{
		xRawMax = xRaw;
	}

	if (yRaw < yRawMin)
	{
		yRawMin = yRaw;
	}
	if (yRaw > yRawMax)
	{
		yRawMax = yRaw;
	}

	if (zRaw < zRawMin)
	{
		zRawMin = zRaw;
	}
	if (zRaw > zRawMax)
	{
		zRawMax = zRaw;
	}
	printf("xRawMin: ");
	cout << xRawMin;
}

int ofApp::ReadAxis(int axisPin)
{
	long reading = 0;
	ardFio.getAnalog(axisPin);
//	Sleep(1);
	for (int i = 0; i < sampleSize; i++)
	{
		reading += ardFio.getAnalog(axisPin);
	}
	return reading / sampleSize;
}
