#pragma once

#include "ofMain.h"
#include "ofEvents.h"
#include "ofArduino.h"
#include "ofxAssimpModelLoader.h"
#include "ofVboMesh.h"
#include <iostream>
#include <vector>

class ofApp : public ofBaseApp {

public:

	void setup();
	void update();
	void draw();

	void keyPressed(int key);
	void keyReleased(int key);

	void mouseMoved(int x, int y);
	void mouseDragged(int x, int y, int button);
	void mousePressed(int x, int y, int button);
	void mouseReleased(int x, int y, int button);
	void mouseEntered(int x, int y);
	void mouseExited(int x, int y);
	void windowResized(int w, int h);
	void dragEvent(ofDragInfo dragInfo);
	void readValues();
	void gotMessage(ofMessage msg);

	ofImage				bgImage;
	ofTrueTypeFont		font;
	ofTrueTypeFont      smallFont;
	ofArduino	ardUno;
	ofArduino	ardFio;
	bool		bSetupArduino;			// flag variable for setting up arduino once

	int mode;

	/* ---- UNO VARIABLES ---- */

	// Analog
	const int soundInput = 0;
	const int pot1Pin = 3;
	const int pot2Pin = 2;
	const int pot3Pin = 1;

	// Digital
	const int but1Pin = 10;
	const int but2Pin = 11;
	const int but3Pin = 12;
	const int calibPin = 13;

	float soundVal;
	float pot1Val;
	float pot2Val;
	float pot3Val;
	float but1Val;
	float but2Val;
	float but3Val;


	/* ---- FIO VARIABLES ---- */
	const int fioLed = 13;
	const int bendOneInput = 0;
	const int bendTwoInput = 1;
	const int bendThreeInput = 2;

	const int xInput = 5;
	const int yInput = 6;
	const int zInput = 7;

	float bendOne;
	float bendTwo;
	float bendThree;

	/* ---- ACCELERATOR VARIABLES ---- */
	bool calib = false;
	// Raw Ranges:
	// initialize to mid-range and allow calibration to
	// find the minimum and maximum for each axis
	int xRawMin = 512;
	int xRawMax = 512;
	int yRawMin = 512;
	int yRawMax = 512;
	int zRawMin = 512;
	int zRawMax = 512;
	// Take multiple samples to reduce noise
	const int sampleSize = 10;
	int ReadAxis(int);
	int xRaw; int yRaw; int zRaw;
	float xAccel, yAccel, zAccel;
	//	int ReadAxis(int axisPin);


	/* ---- MODELS ---- */
	std::vector<std::string> models;


private:

	void setupArduino(const int & version);
	void digitalPinChanged(const int & pinNum);
	void analogPinChanged(const int & pinNum);
	void updateArduino();
	void calibrateAccel(int, int, int);

	string buttonState;
	string potValue;

	// Graphics

	// Hand
	ofMesh mesh;
	ofxAssimpModelLoader model;

	bool bAnimate;
	bool bAnimateMouse;
	float animationPosition;

	// Lights
	ofLight light;
	ofLight pointLight;
	ofLight pointLight2;
	ofLight pointLight3;


	// Variables
	bool bFill;
	bool bWireframe;
	bool bDrawNormals;
	bool bDrawAxes;
	bool bDrawLights;
	bool bInfoText;
	bool bMousePressed;
	bool bSplitFaces;


	float rotateX;
	float rotateY;
	float rotateZ;

	// Dot map thing
	int count;
	float margin;
	float radius;
	int color;
	int alpha;


};